/*
 * ------------------------------------------------------------------------
 *
 *  Copyright (C) 2003 - 2013
 *  University of Konstanz, Germany and
 *  KNIME GmbH, Konstanz, Germany
 *  Website: http://www.knime.org; Email: contact@knime.org
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, Version 3, as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, see <http://www.gnu.org/licenses>.
 *
 *  Additional permission under GNU GPL version 3 section 7:
 *
 *  KNIME interoperates with ECLIPSE solely via ECLIPSE's plug-in APIs.
 *  Hence, KNIME and ECLIPSE are both independent programs and are not
 *  derived from each other. Should, however, the interpretation of the
 *  GNU GPL Version 3 ("License") under any applicable laws result in
 *  KNIME and ECLIPSE being a combined program, KNIME GMBH herewith grants
 *  you the additional permission to use and propagate KNIME together with
 *  ECLIPSE with only the license terms in place for ECLIPSE applying to
 *  ECLIPSE and the GNU GPL Version 3 applying for KNIME, provided the
 *  license terms of ECLIPSE themselves allow for the respective use and
 *  propagation of ECLIPSE together with KNIME.
 *
 *  Additional permission relating to nodes for KNIME that extend the Node
 *  Extension (and in particular that are based on subclasses of NodeModel,
 *  NodeDialog, and NodeView) and that only interoperate with KNIME through
 *  standard APIs ("Nodes"):
 *  Nodes are deemed to be separate and independent programs and to not be
 *  covered works.  Notwithstanding anything to the contrary in the
 *  License, the License does not apply to Nodes, you are not required to
 *  license Nodes under the License, and you are granted a license to
 *  prepare and propagate Nodes, in each case even if such Nodes are
 *  propagated with or for interoperation with KNIME.  The owner of a Node
 *  may freely choose the license terms applicable to such Node, including
 *  when such Node is propagated with or for interoperation with KNIME.
 * ---------------------------------------------------------------------
 *
 * Created on 29.04.2013 by koetter
 */
package edu.cmu.koetter.conceptgraph;

/**
 *
 * @author Tobias Koetter
 */
public class ConceptGraphMinScorer extends AbstractScorer {

    /**
     * @param density density factor
     * @param cueValidity cue validity factor
     * @param size size factor
     */
    public ConceptGraphMinScorer(final int density, final int cueValidity, final int size) {
        super(density, cueValidity, size);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public double computeObjectScore(final ConceptGraphObject object, final int size) {
        double cueValidity = object.getCueValidity();
        double density = object.getOverlap(size);
        return Math.pow(density, getDensity()) * Math.pow(cueValidity, getCueValidity());
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public double computeGraphScore(final Boolean isAspect, final int noOfAspects,
                                    final int noOfMembers, final ScoreCache aspectScores,
                                    final ScoreCache memberScores) {
        final double memberScore;
        final double aspectScore;
        final double sizeScore;
        if (isAspect == null) {
            sizeScore = (noOfMembers * noOfAspects);
            memberScore = memberScores.getScore();
            aspectScore = aspectScores.getScore();
        } else if (isAspect.booleanValue()) {
            sizeScore = (noOfMembers * (noOfAspects - 1));
            memberScore = memberScores.getScore();
            aspectScore = aspectScores.getSecondScore();
        } else {
            sizeScore = (noOfMembers - 1) * noOfAspects;
            memberScore = memberScores.getSecondScore();
            aspectScore = aspectScores.getScore();
        }
        final double score = Math.pow(sizeScore, getSize()) * Math.min(memberScore, aspectScore);
        return score;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean improvesScore(final ScoreCache scores, final double newScore) {
        return scores.getScore() <= newScore;
    }

}
