/*
 * ------------------------------------------------------------------------
 *
 *  Copyright (C) 2003 - 2013
 *  University of Konstanz, Germany and
 *  KNIME GmbH, Konstanz, Germany
 *  Website: http://www.knime.org; Email: contact@knime.org
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, Version 3, as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, see <http://www.gnu.org/licenses>.
 *
 *  Additional permission under GNU GPL version 3 section 7:
 *
 *  KNIME interoperates with ECLIPSE solely via ECLIPSE's plug-in APIs.
 *  Hence, KNIME and ECLIPSE are both independent programs and are not
 *  derived from each other. Should, however, the interpretation of the
 *  GNU GPL Version 3 ("License") under any applicable laws result in
 *  KNIME and ECLIPSE being a combined program, KNIME GMBH herewith grants
 *  you the additional permission to use and propagate KNIME together with
 *  ECLIPSE with only the license terms in place for ECLIPSE applying to
 *  ECLIPSE and the GNU GPL Version 3 applying for KNIME, provided the
 *  license terms of ECLIPSE themselves allow for the respective use and
 *  propagation of ECLIPSE together with KNIME.
 *
 *  Additional permission relating to nodes for KNIME that extend the Node
 *  Extension (and in particular that are based on subclasses of NodeModel,
 *  NodeDialog, and NodeView) and that only interoperate with KNIME through
 *  standard APIs ("Nodes"):
 *  Nodes are deemed to be separate and independent programs and to not be
 *  covered works.  Notwithstanding anything to the contrary in the
 *  License, the License does not apply to Nodes, you are not required to
 *  license Nodes under the License, and you are granted a license to
 *  prepare and propagate Nodes, in each case even if such Nodes are
 *  propagated with or for interoperation with KNIME.  The owner of a Node
 *  may freely choose the license terms applicable to such Node, including
 *  when such Node is propagated with or for interoperation with KNIME.
 * ---------------------------------------------------------------------
 *
 * Created on 06.05.2013 by koetter
 */
package edu.cmu.koetter.conceptgraph;

import org.knime.core.node.util.ButtonGroupEnumInterface;

/**
 *
 * @author koetter
 */
public enum ConceptGraphScore implements ButtonGroupEnumInterface {

    MIN("Minimum", false),
    SUM("Sum", true),
    PROD("Product", false);

    private final String m_title;
    private final boolean m_isDefault;

    private ConceptGraphScore(final String title, final boolean isDefault) {
        m_title = title;
        m_isDefault = isDefault;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getText() {
        return m_title;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getActionCommand() {
        return name();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getToolTip() {
        return null;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isDefault() {
        return m_isDefault;
    }

    /**
     * @param action the action of the scorer
     * @return the {@link ConceptGraphScore} with the given action
     */
    public static ConceptGraphScore get(final String action) {
        return valueOf(action);
    }

    /**
     * @param density density factor
     * @param cueValidity cue validity factor
     * @param size size factor
     * @return the {@link Scorer}
     */
    public Scorer getScorer(final int density, final int cueValidity, final int size) {
        switch (this) {
            case MIN:
                return new ConceptGraphMinScorer(density, cueValidity, size);
            case SUM:
                return new ConceptGraphSumScorer(density, cueValidity, size);
            case PROD:
                return new ConceptGraphProductScorer(density, cueValidity, size);
        }
        throw new RuntimeException("Invalid scorer");
    }
}
