/*
 * ------------------------------------------------------------------------
 *
 *  Copyright (C) 2003 - 2013
 *  University of Konstanz, Germany and
 *  KNIME GmbH, Konstanz, Germany
 *  Website: http://www.knime.org; Email: contact@knime.org
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, Version 3, as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, see <http://www.gnu.org/licenses>.
 *
 *  Additional permission under GNU GPL version 3 section 7:
 *
 *  KNIME interoperates with ECLIPSE solely via ECLIPSE's plug-in APIs.
 *  Hence, KNIME and ECLIPSE are both independent programs and are not
 *  derived from each other. Should, however, the interpretation of the
 *  GNU GPL Version 3 ("License") under any applicable laws result in
 *  KNIME and ECLIPSE being a combined program, KNIME GMBH herewith grants
 *  you the additional permission to use and propagate KNIME together with
 *  ECLIPSE with only the license terms in place for ECLIPSE applying to
 *  ECLIPSE and the GNU GPL Version 3 applying for KNIME, provided the
 *  license terms of ECLIPSE themselves allow for the respective use and
 *  propagation of ECLIPSE together with KNIME.
 *
 *  Additional permission relating to nodes for KNIME that extend the Node
 *  Extension (and in particular that are based on subclasses of NodeModel,
 *  NodeDialog, and NodeView) and that only interoperate with KNIME through
 *  standard APIs ("Nodes"):
 *  Nodes are deemed to be separate and independent programs and to not be
 *  covered works.  Notwithstanding anything to the contrary in the
 *  License, the License does not apply to Nodes, you are not required to
 *  license Nodes under the License, and you are granted a license to
 *  prepare and propagate Nodes, in each case even if such Nodes are
 *  propagated with or for interoperation with KNIME.  The owner of a Node
 *  may freely choose the license terms applicable to such Node, including
 *  when such Node is propagated with or for interoperation with KNIME.
 * ---------------------------------------------------------------------
 *
 * Created on 29.04.2013 by koetter
 */
package edu.cmu.koetter.conceptgraph;

import org.knime.network.core.api.GraphObject;

/**
 *
 * @author Tobias Koetter
 */
public interface Scorer {

    /**
     * @return <code>true</code> if the cue validity is considered
     */
    public boolean useCueValidity();

    /**
     * @param object the {@link GraphObject} to compute the score for
     * @param size the size of its potential concept graph neighbor set
     * @return the score of the given object
     */
    double computeObjectScore(ConceptGraphObject object, int size);

    /**
     * @param isAspect <code>null</code> if no node should be removed, <code>true</code> if the aspect should
     * be removed and <code>false</code> if the member should be removed
     * @param noOfAspects the number of aspects in the concept graph
     * @param noOfMembers the number of members in the concept graph
     * @param aspectScores the {@link ScoreCache} that contains the aspect set score
     * @param memberScores the {@link ScoreCache} that contains the member set score
     * @return the concept graph score
     */
    double computeGraphScore(Boolean isAspect, int noOfAspects, int noOfMembers,
                             ScoreCache aspectScores, ScoreCache memberScores);

    /**
     * @param scores the {@link ScoreCache}
     * @param newScore the new score
     * @return <code>true</code> if the given score improves the total score of the given {@link ScoreCache}
     */
    boolean improvesScore(ScoreCache scores, double newScore);

    public abstract int getSize();

    public abstract int getCueValidity();

    public abstract int getDensity();

}
