/*
 * ------------------------------------------------------------------------
 *
 *  Copyright (C) 2003 - 2013
 *  University of Konstanz, Germany and
 *  KNIME GmbH, Konstanz, Germany
 *  Website: http://www.knime.org; Email: contact@knime.org
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, Version 3, as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, see <http://www.gnu.org/licenses>.
 *
 *  Additional permission under GNU GPL version 3 section 7:
 *
 *  KNIME interoperates with ECLIPSE solely via ECLIPSE's plug-in APIs.
 *  Hence, KNIME and ECLIPSE are both independent programs and are not
 *  derived from each other. Should, however, the interpretation of the
 *  GNU GPL Version 3 ("License") under any applicable laws result in
 *  KNIME and ECLIPSE being a combined program, KNIME GMBH herewith grants
 *  you the additional permission to use and propagate KNIME together with
 *  ECLIPSE with only the license terms in place for ECLIPSE applying to
 *  ECLIPSE and the GNU GPL Version 3 applying for KNIME, provided the
 *  license terms of ECLIPSE themselves allow for the respective use and
 *  propagation of ECLIPSE together with KNIME.
 *
 *  Additional permission relating to nodes for KNIME that extend the Node
 *  Extension (and in particular that are based on subclasses of NodeModel,
 *  NodeDialog, and NodeView) and that only interoperate with KNIME through
 *  standard APIs ("Nodes"):
 *  Nodes are deemed to be separate and independent programs and to not be
 *  covered works.  Notwithstanding anything to the contrary in the
 *  License, the License does not apply to Nodes, you are not required to
 *  license Nodes under the License, and you are granted a license to
 *  prepare and propagate Nodes, in each case even if such Nodes are
 *  propagated with or for interoperation with KNIME.  The owner of a Node
 *  may freely choose the license terms applicable to such Node, including
 *  when such Node is propagated with or for interoperation with KNIME.
 * ---------------------------------------------------------------------
 *
 * Created on 04.04.2013 by koetter
 */
package edu.cmu.koetter.util;

import java.util.BitSet;
import org.knime.core.node.CanceledExecutionException;
import org.knime.core.node.ExecutionMonitor;
import org.knime.network.core.api.KPartiteGraphView;
import org.knime.network.core.api.Partition;
import org.knime.network.core.api.PersistentObject;
import org.knime.network.core.core.exception.PersistenceException;

/**
 *
 * @author Tobias Koetter
 */
public class BitSetBiGraphExtractor extends BiGraphExtractor<BitSet[]> {

    private BitSet[] m_data;

    /**
     * @param view the {@link KPartiteGraphView}
     * @param calculator the {@link EdgeWeightCalculator}
     */
    public BitSetBiGraphExtractor(final KPartiteGraphView<PersistentObject, Partition> view,
                                       final EdgeWeightCalculator calculator) {
        super(view, calculator);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected void init(final int noOfRows, final int noOfCols, final PersistentObject... startNodes)
            throws PersistenceException {
        m_data = new BitSet[noOfRows];
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected void finish(final ExecutionMonitor exec) throws PersistenceException, CanceledExecutionException {
        // nothing to do
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void cleanup() throws PersistenceException {
        //nothing to do
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected void createEdge(final int sourceIdx, final PersistentObject source,
                              final int targetIdx, final PersistentObject target, final double weight) {
        BitSet row = m_data[sourceIdx];
        if (row == null) {
            row = new BitSet(getAspectCount());
            m_data[sourceIdx] = row;
        }
        row.set(targetIdx);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public BitSet[] getData() {
        return m_data;
    }
}