/*
 * ------------------------------------------------------------------------
 *
 *  Copyright (C) 2003 - 2013
 *  University of Konstanz, Germany and
 *  KNIME GmbH, Konstanz, Germany
 *  Website: http://www.knime.org; Email: contact@knime.org
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, Version 3, as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, see <http://www.gnu.org/licenses>.
 *
 *  Additional permission under GNU GPL version 3 section 7:
 *
 *  KNIME interoperates with ECLIPSE solely via ECLIPSE's plug-in APIs.
 *  Hence, KNIME and ECLIPSE are both independent programs and are not
 *  derived from each other. Should, however, the interpretation of the
 *  GNU GPL Version 3 ("License") under any applicable laws result in
 *  KNIME and ECLIPSE being a combined program, KNIME GMBH herewith grants
 *  you the additional permission to use and propagate KNIME together with
 *  ECLIPSE with only the license terms in place for ECLIPSE applying to
 *  ECLIPSE and the GNU GPL Version 3 applying for KNIME, provided the
 *  license terms of ECLIPSE themselves allow for the respective use and
 *  propagation of ECLIPSE together with KNIME.
 *
 *  Additional permission relating to nodes for KNIME that extend the Node
 *  Extension (and in particular that are based on subclasses of NodeModel,
 *  NodeDialog, and NodeView) and that only interoperate with KNIME through
 *  standard APIs ("Nodes"):
 *  Nodes are deemed to be separate and independent programs and to not be
 *  covered works.  Notwithstanding anything to the contrary in the
 *  License, the License does not apply to Nodes, you are not required to
 *  license Nodes under the License, and you are granted a license to
 *  prepare and propagate Nodes, in each case even if such Nodes are
 *  propagated with or for interoperation with KNIME.  The owner of a Node
 *  may freely choose the license terms applicable to such Node, including
 *  when such Node is propagated with or for interoperation with KNIME.
 * ---------------------------------------------------------------------
 *
 * Created on 16.04.2013 by koetter
 */
package edu.cmu.koetter.util;

import java.util.HashMap;
import java.util.Map;
import org.knime.network.core.api.KPartiteGraphView;
import org.knime.network.core.api.Partition;
import org.knime.network.core.api.PersistentObject;
import org.knime.network.core.core.exception.PersistenceException;

/**
 *
 * @author koetter
 */
public abstract class NodeDegreeEdgeWeightCalculator implements EdgeWeightCalculator {

    private final Map<PersistentObject, Integer> m_degreeMap = new HashMap<PersistentObject, Integer>();
    private final Map<PersistentObject, Integer> m_inDegreeMap = new HashMap<PersistentObject, Integer>();
    private final Map<PersistentObject, Integer> m_outDegreeMap = new HashMap<PersistentObject, Integer>();

    /**
     * Constructor.
     */
    protected NodeDegreeEdgeWeightCalculator() {
    }

    /**
     * @param view the {@link KPartiteGraphView}
     * @param node the node to get the degree for
     * @return the node degree
     * @throws PersistenceException if an exception occurs
     */
    protected int getDegree(final KPartiteGraphView<PersistentObject, Partition> view, final PersistentObject node)
            throws PersistenceException {
        final Integer degree = m_degreeMap.get(node);
        if (degree != null) {
            return degree;
        }
        final int nodeDegree = view.getDegree(node);
        m_degreeMap.put(node, Integer.valueOf(nodeDegree));
        return nodeDegree;
    }

    /**
     * @param view the {@link KPartiteGraphView}
     * @param node the node to get the in degree for
     * @return the node in degree
     * @throws PersistenceException if an exception occurs
     */
    protected int getInDegree(final KPartiteGraphView<PersistentObject, Partition> view, final PersistentObject node)
            throws PersistenceException {
        final Integer degree = m_inDegreeMap.get(node);
        if (degree != null) {
            return degree;
        }
        final int nodeDegree = view.getInDegree(node);
        m_inDegreeMap.put(node, Integer.valueOf(nodeDegree));
        return nodeDegree;
    }

    /**
     * @param view the {@link KPartiteGraphView}
     * @param node the node to get the out degree for
     * @return the node out degree
     * @throws PersistenceException if an exception occurs
     */
    protected int getOutDegree(final KPartiteGraphView<PersistentObject, Partition> view, final PersistentObject node)
            throws PersistenceException {
        final Integer degree = m_outDegreeMap.get(node);
        if (degree != null) {
            return degree;
        }
        final int nodeDegree = view.getOutDegree(node);
        m_outDegreeMap.put(node, Integer.valueOf(nodeDegree));
        return nodeDegree;
    }
}
